/*
 * Copyright (c) 2023 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <openthread-system.h>
#include <openthread/ncp.h>
#include <openthread/tasklet.h>
#include "utils/uart.h"

#if defined(CONFIG_RCP_SAMPLE_HCI)
#include "rcp_hci.h"
#endif

LOG_MODULE_REGISTER(coprocessor_sample, CONFIG_OT_COPROCESSOR_LOG_LEVEL);

#define WELCOME_TEXT                                                           \
	"\n\r"                                                                 \
	"\n\r"                                                                 \
	"=========================================================\n\r"        \
	"OpenThread Coprocessor application is now running on NCS.\n\r"        \
	"=========================================================\n\r"

// void otPlatUartReceived(const uint8_t *aBuf, uint16_t aBufLength) { otNcpHdlcReceive(aBuf, aBufLength); }

// void otPlatUartSendDone(void) { otNcpHdlcSendDone(); }

static int NcpSend(const uint8_t *aBuf, uint16_t aBufLength)
{
    otPlatUartSend(aBuf, aBufLength);
    return aBufLength;
}

void otAppNcpInit(otInstance *aInstance)
{
    otPlatUartEnable();

    otNcpHdlcInit(aInstance, NcpSend);
}

int main(void)
{
	LOG_INF(WELCOME_TEXT);

	otInstance *instance;

pseudo_reset:
	otSysInit(0, NULL);   // Initializes OpenThread platform

	instance = otInstanceInitSingle();

	otAppNcpInit(instance);

	while (1)
	{
		otTaskletsProcess(instance);
		otSysProcessDrivers(instance);
	}

	otInstanceFinalize(instance);

#if defined(CONFIG_RCP_SAMPLE_HCI)
	run_hci();
#endif

	goto pseudo_reset;

	return 0;
}
